/*
Copyright (C) 2000  Olivier Cioni

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

You can visit the web site http://www.opale.ovh.org to obtain more informations about this program and/or to contact the coders.
*/


package opale.m2d.ker;

/**
* Cette classe represente les axes 2D X et Y associ  un <code>Pen2D</code> par
l'intermediaire d'un <code>CoordSystem</code>. Elle implemente <code>GObject2D</code>.
* @author OC
* @since Opale-2d 0.1
*/
public class GAxis2D implements GObject2D,Cloneable
{
//membres privs
private InfoAxis info; //Info sur l'axe
private CoordSystem cs;
private boolean affscale,afflabel;
private String xlabel, ylabel; // les labels sur les axes
/**
* Constructeur par defaut
* @param CoordSystem cs, le systeme de coordonnes.
*/
public GAxis2D(CoordSystem cs)
	{
	this.cs = cs;
	info = new InfoAxis();
	affscale = true;
	afflabel = true;
	xlabel = "X";
	ylabel = "Y";
	}

//definition des methodes de l'interface ObjetGraphique2D

public void draw(Pen2D crayon)
	{
	if ( info.isVisible())
	{
		double tmp=cs.getXmin();
		double tick;
	
		crayon.drawLine(cs.getXmin(),0,cs.getXmax(),0);
		crayon.drawLine(0,cs.getYmin(),0,cs.getYmax());
	
		if (afflabel)
			{
			crayon.drawString(xlabel,cs.getXmax(),0,-10,20);
			crayon.drawString(ylabel,0,cs.getYmax(),15,10);
			}
		
		tick = info.getXtickMax();
		if ( (tick = info.getXtickMax()) > 0)
			while (tmp <= cs.getXmax())
			{
			crayon.drawYtick(tmp,0,5);
			if ( affscale) crayon.drawString(Double.toString(tmp),tmp,0,-10,15);
			tmp+=tick;
			}

		tmp = cs.getXmin();
		if ( (tick = info.getXtickMin()) > 0)
			while (tmp <= cs.getXmax())
			{
			crayon.drawYtick(tmp,0,3);
			tmp+=tick;
			}

		tmp =  cs.getYmin();
		if ( (tick = info.getYtickMax()) > 0)
			while (tmp <= cs.getYmax())
				{
				crayon.drawXtick(0,tmp,5);
				if ( affscale) crayon.drawString(Double.toString(tmp),0,tmp,10,5);
				tmp+=tick;
				}
		tmp =  cs.getYmin();
		tick = info.getYtickMin();
		if ( (tick = info.getYtickMin()) > 0)
			while (tmp <= cs.getYmax())
			{
			crayon.drawXtick(0,tmp,3);
			tmp+=tick;
			}
		}
	
	}
	
/**
* Indique si l'affichage de l'echelle sur les axes est effective.
* @param boolean, true = affichage, false = pas d'affichage.
* @since Opale-2d 0.11
*/
public void drawScale(boolean b)
	{
	affscale = b;
	}

/**
* Renvoie l'tat d'affichage  de l'echelle sur les axes.
* @return boolean, true = affichage, false = pas d'affichage.
* @since Opale-2d 0.11
*/
public boolean printScale()
	{
	return affscale;
	}
		
/**
* Indique si l'affichage des labels sur les axes est effectif.
* @param boolean, true = affichage, false = pas d'affichage.
* @since Opale-2d 0.11
*/
public void drawLabel(boolean b)
	{
	afflabel = b;
	}
/**
* Fixe le label sur l'axe des X.
* @param String, le label..
* @since Opale-2d 0.11
*/
public void setXLabel(String lab)
	{
	xlabel = lab;
	}
/**
* Fixe le label sur l'axe des Y.
* @param String, le label..
* @since Opale-2d 0.11
*/
public void setYLabel(String lab)
	{
	ylabel = lab;
	}
		


public InfoObject getInfo()
	{
	return info;
	}

public String toString()
	{
	String s="Axes 2D";
	return s;
	}
/**
* Cre un nouvel objet de mme classe et de mme contenu.
* @return Object un clone de l'objet.
* @exception  OutOfMemoryError s'il n'y a pas assez de mmoire.
* @see        java.lang.Cloneable
*/
public Object clone()
	{
	try
		{
		GAxis2D ax = (GAxis2D) super.clone();
		ax.info = (InfoAxis) info.clone();
		return ax;
		}
	catch (CloneNotSupportedException e) // ne devrait jamais arriver
		{
	    throw new InternalError();
		}
    }

}